/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "frame.h"

void holdemRenderFrameInit() {
  int32_t lWidth, rWidth, height;

  // Prepare the two frame buffers.
  lWidth = HOLDEM_GAME_FRAME_LEFT_WIDTH, rWidth = HOLDEM_GAME_FRAME_RIGHT_WIDTH;
  height = HOLDEM_GAME_FRAME_HEIGHT;
  POKER_STATE.frameLeft = frameBufferCreate(lWidth, height);
  POKER_STATE.frameRight = frameBufferCreate(rWidth, height);
  POKER_STATE.quadLeft = quadCreate(0, 0, 0, 0, 0, lWidth, height, 1, 1);
  POKER_STATE.quadRight = quadCreate(0, 0, 0, 0, 0, rWidth, height, 1, 1);
}

void holdemRenderFrameUpdate() {
  int32_t lWidth, rWidth, height;

  lWidth = HOLDEM_GAME_FRAME_LEFT_WIDTH, rWidth = HOLDEM_GAME_FRAME_RIGHT_WIDTH;
  height = HOLDEM_GAME_FRAME_HEIGHT;
  if((
    POKER_STATE.frameLeft->texture->width == lWidth &&
    POKER_STATE.frameLeft->texture->height == height
  )) return;

  // Recreate frame buffers.
  frameBufferDispose(POKER_STATE.frameLeft);
  frameBufferDispose(POKER_STATE.frameRight);
  POKER_STATE.frameLeft = frameBufferCreate(lWidth, height);
  POKER_STATE.frameRight = frameBufferCreate(rWidth, height);
  quadBuffer(POKER_STATE.quadLeft, 0,
    0, 0, 0, 1,
    lWidth, height, 1, 0,
    0, 0
  );
  quadBuffer(POKER_STATE.quadRight, 0,
    0, 0, 0, 1,
    rWidth, height, 1, 0,
    0, 0
  );
}

void holdemRenderFrameUseLeft() {
  glClearColor(0.3, 0, 0, 1);
  frameBufferUse(POKER_STATE.frameLeft, true);
  cameraPerspective(&POKER_STATE.cameraLeft, 35,
    (
      (float)POKER_STATE.frameLeft->texture->width /
      (float)POKER_STATE.frameLeft->texture->height
    ), 0.2f, 1000.0f
  );
  shaderUseCamera(GAME_STATE.shaderWorld, &POKER_STATE.cameraLeft);
}

void holdemRenderFrameUseRight() {
  glClearColor(0.3, 0.3, 0, 1);
  frameBufferUse(POKER_STATE.frameRight, true);
  cameraPerspective(&POKER_STATE.cameraRight, 45,
    (
      (float)POKER_STATE.frameRight->texture->width /
      (float)POKER_STATE.frameRight->texture->height
    ), 0.2f, 1000.0f
  );
  cameraLookAt(&POKER_STATE.cameraRight, 0, 3, 3, 0, 0, 0);
  shaderUseCamera(GAME_STATE.shaderWorld, &POKER_STATE.cameraRight);
}

void holdemRenderFrameBack() {
  glClearColor(0, 0, 0, 1);
  frameBufferUse(NULL, true);
  cameraOrtho(&GAME_STATE.cameraWorld, 0,
    RENDER_STATE.width, RENDER_STATE.height, 1, 0, 1
  );
  cameraLookAt(&GAME_STATE.cameraWorld, 0, 0, 0.5f, 0, 0, 0);
  shaderUseCamera(GAME_STATE.shaderWorld, &GAME_STATE.cameraWorld);
  shaderUsePosition(GAME_STATE.shaderWorld,
    0, 0, 0, 0, 0, 0
  );
  shaderUseTexture(GAME_STATE.shaderWorld, POKER_STATE.frameLeft->texture);
  primitiveDraw(POKER_STATE.quadLeft, 0, -1);
  shaderUsePosition(GAME_STATE.shaderWorld,
    RENDER_STATE.width - POKER_STATE.frameRight->texture->width,
    0, 0, 0, 0, 0
  );
  shaderUseTexture(GAME_STATE.shaderWorld, POKER_STATE.frameRight->texture);
  primitiveDraw(POKER_STATE.quadRight, 0, -1);
}